module Todo exposing (Model, Msg(..), init, main, update, view)

import Browser
import Html exposing (Html, button, div, input, node, span, text)
import Html.Attributes exposing (attribute, autofocus, checked, class, classList, type_, value)
import Html.Events exposing (onClick, onInput)


main =
    Browser.sandbox { init = init, update = update, view = view }



-- ----------------------------------------------------------------------------------------------------
-- type alias Model =
--     Int
-- init : Model
-- init =
--     0
-- type Msg
--     = Increment
--     | Decrement
-- update : Msg -> Model -> Model
-- update msg model =
--     case msg of
--         Increment ->
--             model + 1
--         Decrement ->
--             model - 1
-- view : Model -> Html Msg
-- view model =
--     div [ class "text-center jumbotron" ]
--         [ button [ onClick Decrement, class "btn btn-primary" ] [ text "-" ]
--         , div [ class "text-capitalize h1" ] [ text (String.fromInt model) ]
--         , button [ onClick Increment, class "btn btn-primary" ] [ text "+" ]
--         ]
-- --------------------------------------------------------------------------------------------------------


type Msg
    = UpdateText String
    | AddTodo
    | Delete Todo
    | ToggleIsDone Int


type alias Todo =
    { id : Int
    , text : String
    , isDone : Bool
    }


type alias Model =
    { text : String
    , todos : List Todo
    , uid : Int
    }


toggle id todo =
    if todo.id == id then
        { todo | isDone = not todo.isDone }

    else
        todo


init : Model
init =
    { uid = 0, text = "", todos = [] }


update : Msg -> Model -> Model
update msg model =
    case msg of
        UpdateText newText ->
            { model | text = newText }

        AddTodo ->
            { model | text = "", uid = model.uid + 1, todos = model.todos ++ [ Todo model.uid model.text False ] }

        Delete todo ->
            { model | todos = List.filter (\t -> t.id /= todo.id) model.todos }

        ToggleIsDone id ->
            { model | todos = List.map (toggle id) model.todos }


view : Model -> Html Msg
view model =
    let
        todoList =
            List.map
                (\todo ->
                    div [ onClick (ToggleIsDone todo.id), class "mt-2" ]
                        [ span [ classList [ ( "text-crossed", todo.isDone ) ] ] [ text todo.text ]
                        , button [ onClick (Delete todo), class "btn btn-danger ml-3" ] [ text "Remove" ]
                        ]
                )
                model.todos
    in
    div [ class "jumbotron text-center" ]
        [ input [ onInput UpdateText, class "mr-3", value model.text, autofocus True ] []
        , button [ onClick AddTodo, class "btn btn-primary" ] [ text "Add Todo " ]
        , div [ class "h3" ] todoList
        ]
